@@ -9,12 +9,13 @@ use anyhow::{bail, Result};
 use bindgen::{
     self,
     callbacks::{EnumVariantCustomBehavior, EnumVariantValue, ParseCallbacks},
+    Builder,
 };
 use quote::quote;
 use std::env;
 use std::fs::File;
 use std::io::{self, Write};
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 use std::process::Command;
 use syn::visit::Visit;
 
@@ -160,6 +161,20 @@ impl ParseCallbacks for HideEnum {
     }
 }
 
+fn get_builder_vmlinux(c_dump_file: impl AsRef<Path>) -> Result<Builder, String> {
+    cfg_if::cfg_if! {
+        if #[cfg(docsrs)] {
+            Ok(bindgen::builder()
+               .use_core()
+               .ctypes_prefix("::cty")
+               .parse_callbacks(Box::new(Callbacks))
+               .header("include/docsrs/vmlinux.h"))
+        } else {
+            bpf_bindgen::get_builder_vmlinux(c_dump_file)
+        }
+    }
+}
+
 fn generate_bindings_vmlinux() -> Result<()> {
     let out_dir = PathBuf::from(env::var("OUT_DIR").unwrap());
     // patterns of whitelist are used to match type names by regex. They are
@@ -188,7 +203,8 @@ fn generate_bindings_vmlinux() -> Result<()> {
         "^sock_flags$", // for enum of SOCK_*
     ];
     let xdp_vars = ["^IPPROTO_.*"];
-    let mut builder = bpf_bindgen::get_builder_vmlinux(out_dir.join("vmlinux.h"))
+
+    let mut builder = get_builder_vmlinux(out_dir.join("vmlinux.h"))
         .or_else(|e| bail!("error on bpf_bindgen::get_builder_vmlinux: {}", e))?
         // // Prevent error E0133: `#[derive]` can't be used on a
         // // `#[repr(packed)]` struct that does not derive Copy
@@ -252,7 +268,7 @@ fn generate_bindings_vmlinux() -> Result<()> {
     create_module(out_dir.join("gen_bindings.rs"), "gen_bindings", &bindings)?;
 
     // Generate bindings of BPF helper variables and convert them into functions
-    let bindings = bpf_bindgen::get_builder_vmlinux(out_dir.join("vmlinux_helpers.h"))
+    let bindings = get_builder_vmlinux(out_dir.join("vmlinux_helpers.h"))
         .or_else(|e| bail!("error on bpf_bindgen::get_builder_vmlinux: {}", e))?
         .header("include/bpf_helpers.h")
         .allowlist_var("^bpf_.*")
@@ -264,14 +280,7 @@ fn generate_bindings_vmlinux() -> Result<()> {
     Ok(())
 }
 
-fn main() {
-    let subscriber = FmtSubscriber::builder()
-        .with_max_level(Level::TRACE)
-        .finish();
-    tracing::subscriber::set_global_default(subscriber).unwrap();
-
-    rerun_if_changed_dir("include");
-
+fn generate_bindings() {
     if get_custom_vmlinux_path().is_some() {
         debug!("Generating bindings with BTF of vmlinux");
         generate_bindings_vmlinux().unwrap();
@@ -290,6 +299,16 @@ fn main() {
     }
 }
 
+fn main() {
+    let subscriber = FmtSubscriber::builder()
+        .with_max_level(Level::TRACE)
+        .finish();
+    tracing::subscriber::set_global_default(subscriber).unwrap();
+
+    rerun_if_changed_dir("include");
+    generate_bindings();
+}
+
 struct RewriteBpfHelpers {
     helpers: Vec<String>,
 }
